
/*     *******************************************************************************
	XVisual GUI Development System for Linux/X-Window
	Copyright (C) 1996 Steve Carrie.

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
        ******************************************************************************* */

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include <X11/xv/xvCore.h>
#include <X11/xv/xvRegion.h>
#include <X11/xv/xvGC.h>

// -----------------------------------

xvGC::xvGC()
{
	alloc=FALSE;
}

// -----------------------------------

xvGC::xvGC(xvCore & object)
{
	init(object);
}

// -----------------------------------

xvGC::~xvGC()
{
	if (alloc) {
		XFreeGC(xvdisplay, maincontext);
		XFreeGC(xvdisplay, backupcontext1);
		XFreeGC(xvdisplay, backupcontext2);
	}
}

// -----------------------------------

void xvGC::init(xvCore & object)
{
	init(object.ID());

}
// -----------------------------------

void xvGC::init(Widget gcw)
{
	xvwidget=gcw;
	xvwindow=XtWindow(gcw);
	xvdisplay=XtDisplay(gcw);
	xvscreen=XtScreen(gcw);

	maincontext=XCreateGC(xvdisplay, xvwindow, 0, NULL);
	backupcontext1=XCreateGC(xvdisplay, xvwindow, 0, NULL);
	backupcontext2=XCreateGC(xvdisplay, xvwindow, 0, NULL);
	xvcontext=maincontext;
	alloc=TRUE;
}

// -----------------------------------

void xvGC::GCFlush()
{
	XFlushGC(xvdisplay, xvcontext);
}

// -----------------------------------

void xvGC::GCSetState(Pixel foreground, Pixel background, int func, Pixel plane_mask)
{
	XSetState(xvdisplay, xvcontext, foreground, background, func, plane_mask);
}

// -----------------------------------

void xvGC::GCSetForeground(Pixel foreground)
{
	if (foreground != -1)
		XSetForeground(xvdisplay, xvcontext, foreground);
}

// ----------------------------------

void xvGC::GCSetBackground(Pixel background)
{
	if (background != -1)
		XSetBackground(xvdisplay, xvcontext, background);
}

// ----------------------------------

void xvGC::GCSetFunction(int func)
{
	XSetFunction(xvdisplay, xvcontext, func);
}

// ----------------------------------

void xvGC::GCSetPlaneMask(Pixel plane_mask)
{
	if (plane_mask != -1)
		XSetPlaneMask(xvdisplay, xvcontext, plane_mask);
}

// ----------------------------------

void xvGC::GCSetLineAttributes(unsigned int line_width, int line_style, int cap_style,
			int join_style)
{
	XSetLineAttributes(xvdisplay, xvcontext, line_width, line_style,
			cap_style, join_style);
}

// ----------------------------------

void xvGC::GCSetDashes(int dash_offset, char dash_list[], int num_elements)
{
	XSetDashes(xvdisplay, xvcontext, dash_offset, dash_list, num_elements);
}

// ----------------------------------

void xvGC::GCSetFillStyle(int fill_style)
{
	XSetFillStyle(xvdisplay, xvcontext, fill_style);
}

// -----------------------------------

void xvGC::GCSetFillRule(int fill_rule)
{
	XSetFillRule(xvdisplay, xvcontext, fill_rule);
}

// -----------------------------------

void xvGC::GCSetTile(Pixmap tile)
{
	XSetTile(xvdisplay, xvcontext, tile);
}

// ------------------------------------

void xvGC::GCSetStipple(Pixmap stipple)
{
	XSetStipple(xvdisplay, xvcontext, stipple);
}

// -------------------------------------

void xvGC::GCSetTSOrigin(int tso_x, int tso_y)
{
	XSetTSOrigin(xvdisplay, xvcontext, tso_x, tso_y);
}

// -------------------------------------

void xvGC::GCSetFont(Font font)
{
	XSetFont(xvdisplay, xvcontext, font);
}

// -------------------------------------

void xvGC::GCSetClipOrigin(int clip_ox, int clip_oy)
{
	XSetClipOrigin(xvdisplay, xvcontext, clip_ox, clip_oy);
}

//  -------------------------------------

void xvGC::GCSetClipMask(Pixmap pix)
{
	XSetClipMask(xvdisplay, xvcontext, pix);
}

// --------------------------------------

void xvGC::GCSetClipRectangles(int clip_ox, int clip_oy, XRectangle rect[],
				int numrec, int xorder)
{
	XSetClipRectangles(xvdisplay, xvcontext, clip_ox, clip_oy, rect, numrec, xorder);
}

// ---------------------------------------

void xvGC::GCSetArcMode(int arc_mode)
{
	XSetArcMode(xvdisplay, xvcontext, arc_mode);
}

// ---------------------------------------

void xvGC::GCSetSubMode(int sub_mode)
{
	XSetSubwindowMode(xvdisplay, xvcontext, sub_mode);
}

// ---------------------------------------

void xvGC::GCSetGraphicsExposures(Boolean gexpose)
{
	XSetGraphicsExposures(xvdisplay, xvcontext, gexpose);
}

// ----------------------------------------

Status xvGC::GCQueryBestSize(int vclass, unsigned int width, unsigned int height, 
					unsigned int  *width_ret, unsigned int *height_ret)
{
	return (XQueryBestSize(xvdisplay, vclass, xvwindow, width, height,
				width_ret, height_ret));
}

// ----------------------------------------

Status xvGC::GCQueryBestTile(unsigned int width, unsigned int height, 
					unsigned int *width_ret, unsigned int *height_ret)

{
	return (XQueryBestTile(xvdisplay, xvwindow, width, height,
				width_ret, height_ret));
}


// ----------------------------------------

Status xvGC::GCQueryBestStipple(unsigned int width, unsigned int height, 
					unsigned int *width_ret, unsigned int *height_ret)
{
	return (XQueryBestStipple(xvdisplay,  xvwindow, width, height,
				width_ret, height_ret));
}

// ----------------------------------------

Window xvGC::WinID()
{
	return(xvwindow);
}

// ----------------------------------------

GC xvGC::GCGetGC()
{
	return(xvcontext);
}

// ----------------------------------------

void xvGC::GCSwitchGC(int which)
{
	switch(which) {
		case XVDEFAULTGC:
			xvcontext=maincontext;
			break;
		case XVAUXGC1:
			xvcontext=backupcontext1;
			break;
		case XVAUXGC2:
			xvcontext=backupcontext2;
			break;
		default:
			xvcontext=maincontext;
	}
}

// -----------------------------------------

void xvGC::GCRegion(xvRegion & r)
{
	XSetRegion(xvdisplay, xvcontext,r.GetRegion());
}

// ------------------------------------------

void xvGC::GCPointerXY(int *x, int *y, int *rx, int *ry, unsigned int *btn)
{
	Window rw, cw;

	XQueryPointer(xvdisplay, xvwindow, &rw, &cw, rx, ry, x, y, btn);
}

//--------------------------------------------  

void xvGC::GCWin2Root(int x, int y, int *tx, int *ty)
{
	Window cw;

	XTranslateCoordinates(xvdisplay, xvwindow, RootWindowOfScreen(xvscreen),
					x, y, tx, ty, &cw);
}

// --------------------------------------------
 
void xvGC::GCRoot2Win(int x, int y, int *tx, int *ty)
{
	Window cw;
	XTranslateCoordinates(xvdisplay, RootWindowOfScreen(xvscreen), xvwindow,
					x, y, tx, ty, &cw);
}

// ---------------------------------------------
